﻿using DuszaRegionalis.commands;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DuszaRegionalis
{
    class Program
    {
        public static string prompt="...";

        /// <summary>
        /// Tartalmazza a kiválasztott mappa elérési útját. Az értékét egy <see cref="ChangeDirectory"/> példány módosíthatja.
        /// Habár a változó statikus (a static main miattt), szépészeti megfontolásokból az értéket az egyes parancsok külön-külön megkapják
        /// </summary>
        public static string repoDir = "";

        /// <summary>
        /// Az a "rejtett" mappa, mely a kiválasztott mappában a korábbi verziókat tartalmazza.
        /// Mivel ez egy elég erős függősége/központja az alkalmazásnak úgy éreztem a legtisztábbnak, ha konstansként tároljuk.
        /// </summary>
        public const string VERSION_CONTROL_DIR = "\\.dusza";

        /// <summary>
        /// Igaz, ha a kiválasztott mappa már inicializált (létezik a verzió követéses mappa <see cref="Program.VERSION_CONTROL_DIR"/>, hamis ha még nem inicializált
        /// </summary>
        public static bool initialized=false;
        
        /// <summary>
        /// Az alkalmazás belépési pontja, mely egy végtelenciklusban pörög. A kilépési lehetőséget az exit parancs megadása adja
        /// A program általános szerkezete, hogy a parancsnak megfelelő példányát hozza létre egy interface-nek <see cref="Command"/>, s meghívja annak "execute" metódusát
        /// Hogy az "execute" mit jelent, a konkrét példánytól függ. Ehhez mérten a commands csomagban található osztályok implementációja a lényeg
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            Console.WriteLine("");
            Console.WriteLine("==========================================");
            Console.WriteLine("Üdvözöllek a verziókövető programban. Ha segítségre van szükséged, futtasd a help parancsot. Kilépéshez írdd be: exit!");
            Console.WriteLine("");

            while (true)
            {
                string fullCommand = beker();
                if(fullCommand == "exit")
                {
                    break;
                }
                Command actualCommand = commandBuilder(fullCommand);
                if (actualCommand == null)
                {
                    hibaKiir("Ismeretlen parancs");
                    continue;
                }
                try
                {
                    actualCommand.execute();

                }catch(Exception e)
                {
                    hibaKiir(e.Message);
                }
                

            }
            info("Köszönöm, hogy használtad a programot.");
            Console.ReadLine();

        }

        /// <summary>
        /// Egyedi, de egységes bekérést szerettem volna, mely emlékeztet a klasszikus parancssori felülethez, ezért ezt külön függvénybe szerveztem.
        /// Folyamatosan láthatjuk tehát a kiválasztott mappát. Úgy is mondhatnám, ahol éppen állunk.
        /// </summary>
        /// <returns></returns>
        private static string beker()
        {
            Console.Write(prompt + ": ");
            return Console.ReadLine();
        }

        /// <summary>
        /// Egyedi, színezett kimenet, mely "információ" jellegű.
        /// </summary>
        /// <param name="info"></param>
        private static void info(string info)
        {
            Console.ForegroundColor = ConsoleColor.Green;
            Console.Write("Info: ");
            Console.ForegroundColor = ConsoleColor.Gray;
            Console.WriteLine(info);
            
        }

        /// <summary>
        /// Egyedi, színezett kimenet, mely valamilyen hibát közöl a felhasználóval
        /// </summary>
        /// <param name="hiba"></param>
        private static void hibaKiir(string hiba)
        {
            Console.ForegroundColor = ConsoleColor.Red;
            Console.Write("Hiba: ");
            Console.ForegroundColor = ConsoleColor.Gray;
            Console.WriteLine(hiba);
        }

        /// <summary>
        /// Visszatér egy Command példánnyal <see cref="Command"/>. A fő feladata tehát a felhasználó bemenetének feldolgozása,
        /// valamint a bemenet alapján a megfelelő példányok legenerálása a konstruktorukkal.
        /// </summary>
        /// <param name="fullCommand">A felhasználó bemenet, vagyis maga a parancs és annak paramétere(i)</param>
        /// <returns>Egy parancs példány. Típusa a felhasználói bemenettől függ</returns>
        private static Command commandBuilder (string fullCommand)
        {
            Command created = null;
            string name = "";
            string commitMessage = "";
            string command = "";
            string options = "";
            if (fullCommand.Split(' ').Length == 1)
            {
                command = fullCommand;
            }
            else
            {
                command = fullCommand.Split(' ')[0];
                for (int i = 1; i < fullCommand.Split(' ').Length; i++)
                {
                    options += fullCommand.Split(' ')[i];
                }
            }
            switch (command)
            {
                case "cd":
                    created = new ChangeDirectory(options);
                    break;
                case "head":
                    created = new ChnangeHead(repoDir, options);
                    break;
                case "list":
                    created = new ListRepositoryData(options,repoDir);
                    break;
                case "commit":
                    Console.Write("Név: ");
                    name = Console.ReadLine();
                    Console.Write("Commit leírás: ");
                    commitMessage = Console.ReadLine();
                    created = new CommitCommand(repoDir, name, commitMessage);
                    break;
                case "init":
                    Console.Write("Név: ");
                    name = Console.ReadLine();
                    created = new Initialize(repoDir, name);
                    break;
                case "help":
                case "-help":
                case "--help":
                    created = new Documentation();
                    break;
            }
            return created;
        }
    }
}
