﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DuszaRegionalis.commands
{
    class ChangeDirectory : Command
    {
        public string options { get; set; }

        /// <summary>
        /// Akkor lehet rá szükség, ha az execute-ot nem szeretnénk futtatni.
        /// Jelen esetben ez akkor lehetséges, ha csak a dokumentáció érdekel.
        /// </summary>
        public ChangeDirectory()
        {

        }

        /// <summary>
        /// Létrehoz egy mappaváltásért felelős példányt, melynek függősége, hogy "hova" szeretnénk elmenni a fájlrendszeren
        /// </summary>
        /// <param name="options">Egy mappa elérési útja, mely az execute után a program "aktuális" mappája lesz, melyeken a további akciók végrehajthatók</param>
        public ChangeDirectory(string options)
        {
            this.options = options;
        }

        /// <summary>
        /// A mappaváltás végrehajtása nem más, mint a kiválasztott mappa létének ellenőrzése, valamint letárolása.
        /// Hogy a felhasználó kaphasson információt "hol van éppen", ezért az a konzolban meg is jelenik <see cref="Program.Main(string[])"/>
        /// Annak a kiértékelése is itt történik, hogy a kiválasztott mappa inicializált-e.
        /// </summary>
        void Command.execute()
        {
            if(String.IsNullOrEmpty(options))
            {
                throw new Exception("Nem lehet mappát választani további paraméter nélkül. Próbáld megint futtatni a parancsot, majd szóköz után az elérési utat.");
            }
            
            if (Directory.Exists(options))
            {
                Program.repoDir = options;
                Program.prompt = options;
            }
            else
            {
                throw new Exception("A megadott mappa nem létezik");
            }
            if (Directory.Exists(options + Program.VERSION_CONTROL_DIR))
            {
                Program.prompt += "(inicializalt)";
                Program.initialized = true;
            }
            else
            {
                Program.prompt += "(inicializálatlan)";
                Program.initialized = false;
            }
        }

        /// <summary>
        /// A cd parancs dokumentációját adja meg.
        /// </summary>
        /// <returns>A cd parancs dokumentációjával</returns>
        string Command.getHelp()
        {
            return "Használat: cd <Elérési út>\nEzzel a paranccsal tudsz mappát választani, amiben később végre tudok hajtani a verziókövetéshez használt parancsokat";
        }
    }
}
