﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DuszaRegionalis.commands
{
    class ChnangeHead : CommitCommand, Command
    {
        public string commitId { get; set; }

        /// <summary>
        /// Akkor lehet rá szükség, ha az execute-ot nem szeretnénk futtatni.
        /// Jelen esetben ez akkor lehetséges, ha csak a dokumentáció érdekel.
        /// </summary>
        public ChnangeHead()
        {

        }

        /// <summary>
        /// Létrehoz egy aktív commit megváltoztatásáért felelős példányt, melynek függősége, hogy melyik repositorynak szeretnék megváltoztatni,
        /// valamint, hogy melyik commit állapotára.
        /// </summary>
        /// <param name="repoDir">A repository elérési útja</param>
        /// <param name="commitId">A commit azonosítója, mely állapotára hozzuk a repositoryt</param>
        public ChnangeHead(string repoDir, string commitId)
        {
            this.repoDir = repoDir;
            this.commitId = commitId;
        }

        /// <summary>
        /// Az aktív commit megváltoztatása, a head parancs logikája.
        /// Ez annyit tesz, hogy kitörli a mappa tartalmát a verziókövetéses mappa <see cref="Program.VERSION_CONTROL_DIR"/> kivételével,
        /// ezek után bemásolja a felhasználó által megadott commitnak megfelelő mappa tartalmát az eredeti mappába.
        /// </summary>
        void Command.execute()
        {
            if (!Program.initialized) throw new Exception("A parancs nem használható inicializálatlan mappák esetében."); 
            string commitPath = findPathForGivenCommitId();
            if (String.IsNullOrEmpty(commitPath))
            {
                throw new Exception("A megadott commit nem található.");
            }
            deleteAllFromRepository();
            copyDirectory(commitPath, repoDir);
            Encoding utf8WithoutBom = new UTF8Encoding(false);
            File.WriteAllText(repoDir + Program.VERSION_CONTROL_DIR +"\\head.txt", commitId, utf8WithoutBom);

        }

        /// <summary>
        /// Megadja a paraméterben megadott commithoz tartozó teljes elérési utat. Amennyiben ilyen commit nem létezik, üres sringet ad.
        /// </summary>
        /// <returns>A teljes elérési útja a commitnak, vagy üres string ha nincs ilyen commit.</returns>
        private string findPathForGivenCommitId()
        {
            DirectoryInfo di = new DirectoryInfo(repoDir + Program.VERSION_CONTROL_DIR + "\\");

            foreach (DirectoryInfo dir in di.EnumerateDirectories())
            {
                if (dir.Name == commitId)
                {
                    return dir.FullName;
                }
            }
            return "";
        }

        /// <summary>
        /// Kitörli a kiválasztott mappa elemeit. Csak a fileokkal foglalkozik, azaz semmilyen mappát nem töröl.
        /// </summary>
        void deleteAllFromRepository()
        {
            DirectoryInfo di = new DirectoryInfo(repoDir);
            foreach (FileInfo file in di.EnumerateFiles())
            {
                file.Delete();
            }
        }

        /// <summary>
        /// A head parancs dokumentációját adja meg.
        /// </summary>
        /// <returns>A head parancs dokumentációjával</returns>
        string Command.getHelp()
        {
            return "Használat: head <commitId>\nEzzel a paranccsal tudod megváltoztatni az aktív commitot.";
        }
    }
}
