﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DuszaRegionalis.commands
{
    class Initialize : Command
    {
        public string repoDir { get; set; }
        public string author { get; set; }

        /// <summary>
        /// Akkor lehet rá szükség, ha az execute-ot nem szeretnénk futtatni.
        /// Jelen esetben ez akkor lehetséges, ha csak a dokumentáció érdekel.
        /// </summary>
        public Initialize()
        {

        }

        /// <summary>
        /// Létrehoz egy inicializálásért felelős példányt, melynek függősége, hogy melyik repositoryt szeretnénk inicializálni és milyen néven
        /// </summary>
        /// <param name="repoDir">A repository elérési útja</param>
        /// <param name="author">Az inicalizáló neve, mely a commit részleteibe fog bekerülni <see cref="CommitCommand"/></param>
        public Initialize(string repoDir, string author)
        {
            this.repoDir = repoDir;
            this.author = author;
        }

        /// <summary>
        /// Az init parancs logikája. Inicializálja a kiválasztott mappát.
        /// Ez annyit tesz, hogy létehozza a verziókövetéses mappát <see cref="Program.VERSION_CONTROL_DIR"/>, abban a head filet, valamint az első commitot.
        /// Természetesen már inicializált mappa esetén hibát dob. Inicializálatlan esetében pedig a prompt-ot is átírja, amit a bekérésnél használ a main függvény <see cref="Program.beker"/>
        /// </summary>
        void Command.execute()
        {
            if (Directory.Exists(repoDir + Program.VERSION_CONTROL_DIR))
            {
                throw new Exception("A mappa már inicializált");
            }

            Directory.CreateDirectory(repoDir + Program.VERSION_CONTROL_DIR);
            Program.prompt = repoDir + "(inicializalt)";
            Program.initialized = true;
            Command firstCommit = new CommitCommand(repoDir, author, "Init commit");
            firstCommit.execute();
            

        }

        /// <summary>
        /// Az init parancs dokumentációját adja meg.
        /// </summary>
        /// <returns>Az init parancs dokumentációjával</returns>
        string Command.getHelp()
        {
            return "Használat: init\nEzzel a paranccsal tudsz egy mappát inicializálni (az aktuálisat).";
        }
    }
}
